/*!\file:  objectivefunctionC
 * \brief  objective function that returns a misfit, for a certain parameter.
 */ 

/*include files: {{{1*/
#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "../toolkits/toolkits.h"
#include "../objects/objects.h"
#include "../shared/shared.h"
#include "../EnumDefinitions/EnumDefinitions.h"
#include "../solvers/solvers.h"
#include "./solutions.h"
#include "../modules/modules.h"
#include "../include/include.h"
/*}}}*/

double objectivefunctionC(double search_scalar,OptArgs* optargs){

	int i;  
	
	/*output: */
	double J;
	
	/*parameters: */
	FemModel  *femmodel       = NULL;
	int        n;
	double    *optscal        = NULL;
	double    *responses      =NULL;
	int        control_type;
	int        solution_type;
	int        analysis_type;
	bool       isstokes       = false;
	bool       conserve_loads = true;

	/*Recover finite element model: */
	femmodel=optargs->femmodel;

	/*Recover parameters: */
	n=optargs->n;

	femmodel->parameters->FindParam(&optscal,NULL,OptScalEnum);
	femmodel->parameters->FindParam(&responses,NULL,CmResponsesEnum);
	femmodel->parameters->FindParam(&isstokes,IsStokesEnum);
	femmodel->parameters->FindParam(&control_type,ControlTypeEnum);
	femmodel->parameters->FindParam(&analysis_type,AnalysisTypeEnum);
	femmodel->parameters->FindParam(&solution_type,SolutionTypeEnum);

	/*set analysis type to compute velocity: */
	if (solution_type==SteadystateSolutionEnum || solution_type==DiagnosticSolutionEnum){
		femmodel->SetCurrentConfiguration(DiagnosticHorizAnalysisEnum);
	}
	else if (solution_type==BalancedthicknessSolutionEnum){
		femmodel->SetCurrentConfiguration(BalancedthicknessAnalysisEnum);
	}
	else{
		ISSMERROR("Solution %s not implemented yet",EnumToString(solution_type));
	}

	/*update parameter according to scalar: */ //false means: do not save control
	InputControlUpdatex(femmodel->elements,femmodel->nodes,femmodel->vertices,femmodel->loads,femmodel->materials,femmodel->parameters,search_scalar*optscal[n],false);

	/*Run diagnostic with updated inputs: */
	if (solution_type==SteadystateSolutionEnum){
		diagnostic_core(femmodel);	//We need a 3D velocity!! (vz is required for the next thermal run)
	}
	else if (solution_type==DiagnosticSolutionEnum){
		solver_diagnostic_nonlinear(femmodel,conserve_loads); 
	}
	else if (solution_type==BalancedthicknessSolutionEnum){
		solver_linear(femmodel); 
	}
	else{
		ISSMERROR("Solution %s not implemented yet",EnumToString(solution_type));
	}

	/*Compute misfit for this velocity field.*/
	CostFunctionx( &J, femmodel->elements,femmodel->nodes, femmodel->vertices,femmodel->loads, femmodel->materials, femmodel->parameters,(int)responses[n]);

	/*Free ressources:*/
	xfree((void**)&responses);
	xfree((void**)&optscal);
	return J;
}
